package org.klomp.eclipse.fsize;

import java.text.MessageFormat;

import org.eclipse.core.runtime.IPluginDescriptor;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;

import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPreferencePage;

import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.preference.PreferencePage;

public class FSDPreferencePage
    extends PreferencePage
    implements IWorkbenchPreferencePage {

    public final static String PREF_FORMAT_STRING = "pref_format_string";
    public final static String PREF_LOCATION = "pref_location";
    public final static String PREF_BYTES_STRING = "pref_bytes_string";
    public final static String PREF_KILO_STRING = "pref_kilo_string";
    public final static String PREF_MEGA_STRING = "pref_mega_string";
    public final static String PREF_GIGA_STRING = "pref_giga_string";
    public final static String PREF_KILO_FACTOR = "pref_kilo_factor";

    private Text formatString;
    private Text bytesString;
    private Text kiloString;
    private Text megaString;
    private Text gigaString;

    private Button use1000s;
    private Button use1024s;

    private Button asPrefix;
    private Button asSuffix;

    public FSDPreferencePage() {
        setDescription(
            FileSizeDecoratorPlugin.getResourceString(
                "%PreferencePage.description"));
    }

    protected Control createContents(Composite parent) {
        Composite table = createTable(parent, 3, 1);

        IPluginDescriptor strings =
            FileSizeDecoratorPlugin.getPluginDescriptor();

        createLabel(
            table,
            strings.getResourceString("%PreferencePage.formatStringLabel"),
            1);
        formatString = createTextField(table, 2);
        formatString.addModifyListener(new ModifyListener() {
            public void modifyText(ModifyEvent event) {
                // Parse the timeout value
                try {
                    new MessageFormat(formatString.getText());
                    setErrorMessage(null);
                    setValid(true);
                } catch (IllegalArgumentException e) {
                    setErrorMessage(e.getMessage());
                    setValid(false);
                }
            }
        });
        createLabel(
            table,
            strings.getResourceString(
                "%PreferencePage.formatStringDescription"),
            3,
            2);
        createSeparator(table, 3);
        createLabel(
            table,
            strings.getResourceString("%PreferencePage.locationLabel"),
            1);
        Composite locationTable = createTable(table, 2, 2);
        asPrefix =
            createRadioButton(
                locationTable,
                strings.getResourceString("%PreferencePage.prefixValueLabel"));
        asSuffix =
            createRadioButton(
                locationTable,
                strings.getResourceString("%PreferencePage.suffixValueLabel"));
        //createKiloFactor(table);

        createLabel(
            table,
            strings.getResourceString("%PreferencePage.kiloFactorLabel"),
            1);
        Composite kiloFactorTable = createTable(table, 2, 2);
        use1024s = createRadioButton(kiloFactorTable, "1024");
        use1000s = createRadioButton(kiloFactorTable, "1000");

        createLabel(table, "", 3);

        createLabel(
            table,
            strings.getResourceString("%PreferencePage.bytesUnitLabel"),
            1);
        bytesString = createTextField(table, 1);
        createLabel(table, "", 1);
        createLabel(
            table,
            strings.getResourceString("%PreferencePage.kiloUnitLabel"),
            1);
        kiloString = createTextField(table, 1);
        createLabel(table, "", 1);
        createLabel(
            table,
            strings.getResourceString("%PreferencePage.megaUnitLabel"),
            1);
        megaString = createTextField(table, 1);
        createLabel(table, "", 1);
        createLabel(
            table,
            strings.getResourceString("%PreferencePage.gigaUnitLabel"),
            1);
        gigaString = createTextField(table, 1);
        createLabel(table, "", 1);

        setControlValuesFromStore();

        return table;
    }

    private Composite createTable(
        Composite parent,
        int numColumns,
        int colSpan) {
        Composite table = new Composite(parent, SWT.NULL);

        // Table grid, special number of columns, number of
        // rows dynamically determined.
        GridLayout layout = new GridLayout();
        layout.numColumns = numColumns;
        layout.marginWidth = 0;
        layout.marginHeight = 0;
        table.setLayout(layout);

        GridData data = new GridData();
        // Align by filling available space with table
        data.verticalAlignment = GridData.FILL;
        data.horizontalAlignment = GridData.FILL;
        data.horizontalSpan = colSpan;
        table.setLayoutData(data);
        return table;
    }

    private Label createLabel(Composite parent, String text, int colSpan) {
        Label label = new Label(parent, SWT.LEFT);
        label.setText(text);
        GridData data = new GridData();
        data.horizontalSpan = colSpan;
        data.horizontalAlignment = GridData.FILL;
        label.setLayoutData(data);
        return label;
    }

    private Label createSeparator(Composite parent, int colSpan) {
        Label label = new Label(parent, SWT.SEPARATOR | SWT.HORIZONTAL);
        GridData data = new GridData();
        data.horizontalSpan = colSpan;
        data.horizontalAlignment = GridData.FILL;
        label.setLayoutData(data);
        return label;
    }

    private Label createLabel(
        Composite parent,
        String text,
        int colSpan,
        int rowSpan) {

        Label label = new Label(parent, SWT.LEFT);
        label.setText(text);
        GridData data = new GridData();
        data.horizontalSpan = colSpan;
        data.verticalSpan = rowSpan;
        data.horizontalAlignment = GridData.FILL;
        label.setLayoutData(data);
        return label;
    }

    private Text createTextField(Composite group, int colSpan) {
        Text text = new Text(group, SWT.BORDER);
        GridData data = new GridData(GridData.FILL_HORIZONTAL);
        data.horizontalSpan = colSpan;
        text.setLayoutData(data);
        return text;
    }

    protected Button createRadioButton(Composite parent, String label) {
        Button button = new Button(parent, SWT.RADIO | SWT.LEFT);
        button.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
        button.setText(label);
        return button;
    }

    public void init(IWorkbench workbench) {
    }

    protected IPreferenceStore doGetPreferenceStore() {
        return FileSizeDecoratorPlugin.getDefault().getPreferenceStore();
    }

    public boolean performOk() {

        IPreferenceStore store = getPreferenceStore();

        store.setValue(PREF_FORMAT_STRING, formatString.getText());
        store.setValue(PREF_BYTES_STRING, bytesString.getText());
        store.setValue(PREF_KILO_STRING, kiloString.getText());
        store.setValue(PREF_MEGA_STRING, megaString.getText());
        store.setValue(PREF_GIGA_STRING, gigaString.getText());
        store.setValue(PREF_KILO_FACTOR, use1024s.getSelection() ? 1024 : 1000);
        store.setValue(
            PREF_LOCATION,
            asPrefix.getSelection()
                ? FileSizeDecoratorPlugin.LOCATION_PREFIX
                : FileSizeDecoratorPlugin.LOCATION_SUFFIX);

        FileSizeDecoratorPlugin plugin = FileSizeDecoratorPlugin.getDefault();

        plugin.setFormatString(store.getString(PREF_FORMAT_STRING));
        plugin.setLocation(store.getInt(PREF_LOCATION));
        plugin.setBytesString(store.getString(PREF_BYTES_STRING));
        plugin.setKiloString(store.getString(PREF_KILO_STRING));
        plugin.setMegaString(store.getString(PREF_MEGA_STRING));
        plugin.setGigaString(store.getString(PREF_GIGA_STRING));
        plugin.setKiloFactor(store.getInt(PREF_KILO_FACTOR));

        FileSizeDecorator.refresh();

        plugin.savePluginPreferences();
        return true;
    }

    protected void performDefaults() {
        IPreferenceStore store = getPreferenceStore();

        formatString.setText(store.getDefaultString(PREF_FORMAT_STRING));
        asPrefix.setSelection(
            store.getDefaultInt(PREF_LOCATION)
                == FileSizeDecoratorPlugin.LOCATION_PREFIX);
        asSuffix.setSelection(
            store.getDefaultInt(PREF_LOCATION)
                == FileSizeDecoratorPlugin.LOCATION_SUFFIX);
        bytesString.setText(store.getDefaultString(PREF_BYTES_STRING));
        kiloString.setText(store.getDefaultString(PREF_KILO_STRING));
        megaString.setText(store.getDefaultString(PREF_MEGA_STRING));
        gigaString.setText(store.getDefaultString(PREF_GIGA_STRING));
        use1000s.setSelection(store.getDefaultInt(PREF_KILO_FACTOR) == 1000);
        use1024s.setSelection(store.getDefaultInt(PREF_KILO_FACTOR) != 1000);
    }

    private void setControlValuesFromStore() {
        IPreferenceStore store = getPreferenceStore();

        formatString.setText(store.getString(PREF_FORMAT_STRING));
        asPrefix.setSelection(
            store.getInt(PREF_LOCATION)
                == FileSizeDecoratorPlugin.LOCATION_PREFIX);
        asSuffix.setSelection(
            store.getInt(PREF_LOCATION)
                == FileSizeDecoratorPlugin.LOCATION_SUFFIX);
        bytesString.setText(store.getString(PREF_BYTES_STRING));
        kiloString.setText(store.getString(PREF_KILO_STRING));
        megaString.setText(store.getString(PREF_MEGA_STRING));
        gigaString.setText(store.getString(PREF_GIGA_STRING));
        use1000s.setSelection(store.getInt(PREF_KILO_FACTOR) == 1000);
        use1024s.setSelection(store.getInt(PREF_KILO_FACTOR) != 1000);
    }

}